@@ -4,6 +4,7 @@
  * Returned values are in kiB */
 
 #include <errno.h>
+#include <signal.h>
 #include <stddef.h> // for size_t
 #include <stdio.h>
 #include <stdlib.h>
@@ -73,8 +74,11 @@ meminfo_t parse_meminfo()
     char buf[8192] = { 0 };
     meminfo_t m = { 0 };
 
-    if (fd == NULL)
-        fd = fopen("/proc/meminfo", "r");
+    if (fd == NULL) {
+        char buf[PATH_LEN] = { 0 };
+        snprintf(buf, sizeof(buf), "%s/%s", procdir_path, "meminfo");
+        fd = fopen(buf, "r");
+    }
     if (fd == NULL) {
         fatal(102, "could not open /proc/meminfo: %s\n", strerror(errno));
     }
@@ -121,24 +125,50 @@ meminfo_t parse_meminfo()
 
 bool is_alive(int pid)
 {
-    char buf[256];
+    // whole process group (-g flag)?
+    if (pid < 0) {
+        // signal 0 does nothing, but we do get an error when the process
+        // group does not exist.
+        int res = kill(pid, 0);
+        if (res == 0) {
+            return true;
+        }
+        return false;
+    }
+
+    char buf[PATH_LEN] = { 0 };
     // Read /proc/[pid]/stat
-    snprintf(buf, sizeof(buf), "/proc/%d/stat", pid);
+    snprintf(buf, sizeof(buf), "%s/%d/stat", procdir_path, pid);
     FILE* f = fopen(buf, "r");
     if (f == NULL) {
         // Process is gone - good.
         return false;
     }
+
     // File content looks like this:
     // 10751 (cat) R 2663 10751 2663[...]
-    char state;
-    int res = fscanf(f, "%*d %*s %c", &state);
-    int fscanf_errno = errno;
+    // File may be bigger than 256 bytes, but we only need the first 20 or so.
+    memset(buf, 0, sizeof(buf));
+    size_t len = fread(buf, 1, sizeof(buf), f);
+    bool read_error = ferror(f) || len == 0;
     fclose(f);
-    if (res < 1) {
-        warn("is_alive: fscanf() failed: %s\n", strerror(fscanf_errno));
+    if (read_error) {
+        warn("%s: fread failed: %s\n", __func__, strerror(errno));
         return false;
     }
+
+    // Find last ")" by searching from the end
+    int i = sizeof(buf) - 1;
+    for (; i >= 0; i--) {
+        if (buf[i] == ')')
+            break;
+    }
+    if (i <= 0 || i + 2 >= (int)sizeof(buf)) {
+        warn("%s: could not find closing bracket\n", __func__);
+        return false;
+    }
+    char state = buf[i + 2];
+
     debug("process state: %c\n", state);
     if (state == 'Z') {
         // A zombie process does not use any memory. Consider it dead.
@@ -155,7 +185,7 @@ bool is_alive(int pid)
 static int read_proc_file_integer(const int pid, const char* name, int* out)
 {
     char path[PATH_LEN] = { 0 };
-    snprintf(path, sizeof(path), "/proc/%d/%s", pid, name);
+    snprintf(path, sizeof(path), "%s/%d/%s", procdir_path, pid, name);
     FILE* f = fopen(path, "r");
     if (f == NULL) {
         return -errno;
@@ -198,7 +228,7 @@ int get_oom_score_adj(const int pid, int* out)
 int get_comm(int pid, char* out, size_t outlen)
 {
     char path[PATH_LEN] = { 0 };
-    snprintf(path, sizeof(path), "/proc/%d/comm", pid);
+    snprintf(path, sizeof(path), "%s/%d/comm", procdir_path, pid);
     FILE* f = fopen(path, "r");
     if (f == NULL) {
         return -errno;
@@ -244,7 +274,7 @@ long long get_vm_rss_kib(int pid)
     char path[PATH_LEN] = { 0 };
 
     // Read VmRSS from /proc/[pid]/statm (in pages)
-    snprintf(path, sizeof(path), "/proc/%d/statm", pid);
+    snprintf(path, sizeof(path), "%s/%d/statm", procdir_path, pid);
     FILE* f = fopen(path, "r");
     if (f == NULL) {
         return -errno;
