@@ -9,6 +9,7 @@ import (
 // #include "meminfo.h"
 // #include "kill.h"
 // #include "msg.h"
+// #include "globals.h"
 import "C"
 
 func parse_term_kill_tuple(optarg string, upper_limit int) (error, float64, float64) {
@@ -45,7 +46,7 @@ func kill_process() {
 	var args C.poll_loop_args_t
 	var victim C.procinfo_t
 	victim.pid = 1
-	C.kill_process(&args, 0, victim)
+	C.kill_process(&args, 0, &victim)
 }
 
 func get_oom_score(pid int) int {
@@ -68,3 +69,8 @@ func get_comm(pid int) (int, string) {
 	res := C.get_comm(C.int(pid), cstr, 256)
 	return int(res), C.GoString(cstr)
 }
+
+func procdir_path(str string) {
+	cstr := C.CString(str)
+	C.procdir_path = cstr
+}
