@@ -1,6 +1,7 @@
 package earlyoom_testsuite
 
 import (
+	"io/ioutil"
 	"os"
 	"strings"
 	"syscall"
@@ -83,6 +84,46 @@ func TestIsAlive(t *testing.T) {
 	}
 }
 
+func TestIsAliveMock(t *testing.T) {
+	mockProcdir, err := ioutil.TempDir("", t.Name())
+	if err != nil {
+		t.Fatal(err)
+	}
+	procdir_path(mockProcdir)
+	defer procdir_path("/proc")
+
+	if err := os.Mkdir(mockProcdir+"/100", 0700); err != nil {
+		t.Fatal(err)
+	}
+
+	testCases := []struct {
+		content string
+		res     bool
+	}{
+		{"144815 (bash) S 17620 144815 144815 34817 247882 4194304 20170 1855121 1 3321 28 46 3646 3366 20 0 1 0 10798280 237576192 1065 18446744073709551615 94174652813312 94174653706789 140724247111872 0 0 0 65536 3686404 1266761467 0 0 0 17 0 0 0 9 0 0 94174653946928 94174653994640 94174663303168 140724247119367 140724247119377 140724247119377 140724247121902 0",
+			true}, // full string from actual system
+		{"123 (bash) R 123 123 123", true}, // truncated for brevity
+		{"123 (bash) Z 123 123 123", false},
+		// hostile process names that try to fake "I am dead"
+		{"123 (foo) Z ) R 123 123 123", true},
+		{"123 (foo) Z) R 123 123 123", true},
+		{"123 (foo)Z ) R 123 123 123", true},
+		{"123 (foo)\nZ\n) R 123 123 123", true},
+		{"123 (foo)\tZ\t) R 123 123 123", true},
+		{"123 (foo)  Z   ) R 123 123 123", true},
+	}
+
+	for _, tc := range testCases {
+		statFile := mockProcdir + "/100/stat"
+		if err := ioutil.WriteFile(statFile, []byte(tc.content), 0600); err != nil {
+			t.Fatal(err)
+		}
+		if is_alive(100) != tc.res {
+			t.Errorf("have=%v, want=%v for /proc/100/stat=%q", is_alive(100), tc.res, tc.content)
+		}
+	}
+}
+
 func Test_fix_truncated_utf8(t *testing.T) {
 	// From https://gist.github.com/w-vi/67fe49106c62421992a2
 	str := "___😀∮ E⋅da = Q,  n → ∞, 𐍈∑ f(i) = ∏ g(i)"
